/*******************************************************************************
* File Name:   main.c
*
* Description: This code demonstrates how to use the Imagimob generated
*              AI model on the XMC72 EVK device
*
* Related Document: See README.md
*
*
*******************************************************************************
* Copyright 2021-2024, Cypress Semiconductor Corporation (an Infineon company) or
* an affiliate of Cypress Semiconductor Corporation.  All rights reserved.
*
* This software, including source code, documentation and related
* materials ("Software") is owned by Cypress Semiconductor Corporation
* or one of its affiliates ("Cypress") and is protected by and subject to
* worldwide patent protection (United States and foreign),
* United States copyright laws and international treaty provisions.
* Therefore, you may use this Software only as provided in the license
* agreement accompanying the software package from which you
* obtained this Software ("EULA").
* If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
* non-transferable license to copy, modify, and compile the Software
* source code solely for use in connection with Cypress's
* integrated circuit products.  Any reproduction, modification, translation,
* compilation, or representation of this Software except as specified
* above is prohibited without the express written permission of Cypress.
*
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
* reserves the right to make changes to the Software without notice. Cypress
* does not assume any liability arising out of the application or use of the
* Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use in any products where a malfunction or
* failure of the Cypress product may reasonably be expected to result in
* significant property damage, injury or death ("High Risk Product"). By
* including Cypress's product in a High Risk Product, the manufacturer
* of such system or application assumes all risk of such use and in doing
* so agrees to indemnify Cypress against all liability.
*******************************************************************************/

/*******************************************************************************
* Header Files
*******************************************************************************/
#include "cyhal.h"
#include "cybsp.h"

#include "cy_retarget_io.h"

/* Import model from Imagimob */
#include "model.h"

/* Inlcude pcm_samples array. This header defines the array, so include it only once */
#include "pcm_samples.h"

/******************************************************************************
* Macros
*******************************************************************************/
#define SAMPLE_RATE     (16000)

/*******************************************************************************
* Global Variables
*******************************************************************************/


/*******************************************************************************
* Function Prototypes
*******************************************************************************/


/*******************************************************************************
* Function Definitions
*******************************************************************************/

/*******************************************************************************
* Function Name: main
*********************************************************************************
* Summary:
* Entry point of application. Contains whole code for this project.
* Initializes device and board, enables UART printing via printf(),
* reads PCM data from pcm_samples array and puts it continuously into the
* Imagimob model. Status information is printed to a serial terminal.
*
* Parameters:
*  void
*
* Return:
*  int
*
*******************************************************************************/
int main(void)
{
    cy_rslt_t result;

#if defined (CY_DEVICE_SECURE)
    cyhal_wdt_t wdt_obj;

    /* Clear watchdog timer so that it doesn't trigger a reset */
    result = cyhal_wdt_init(&wdt_obj, cyhal_wdt_get_max_timeout_ms());
    CY_ASSERT(CY_RSLT_SUCCESS == result);
    cyhal_wdt_free(&wdt_obj);
#endif

    /* Initialize the device and board peripherals */
    result = cybsp_init();

    /* Board init failed. Stop program execution */
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }

    /* Enable global interrupts */
    __enable_irq();

    /* Initialize retarget-io to use the debug UART port */
    result = cy_retarget_io_init_fc(CYBSP_DEBUG_UART_TX, CYBSP_DEBUG_UART_RX,
            CYBSP_DEBUG_UART_CTS,CYBSP_DEBUG_UART_RTS,CY_RETARGET_IO_BAUDRATE);

    /* Command sequence to clear terminal */
    printf("\x1b[2J\x1b[H");

    /* Initialize Imagimob model */
    IMAI_init();

    /* Input and output of model */
    float sample = 0.f;
    float classifications[2];

    /* Index to keep track where we are in PCM data */
    uint32_t pcm_idx = 0;

    /* Counter to keep track at which time in seconds we are in PCM data */
    uint32_t samples_sec_cnt = 0;
    uint32_t seconds = 0;

    for (;;)
    {
        /* Get PCM sample and convert to floating point */
        sample = (float) pcm_samples[pcm_idx];

        /* Scale sample to range [-1.f, 1.f]. Current assumption is that full spectrum of 16-bit is used
         * (2^15 = 32768, 15 and not 16 since data is two-complement). If this is not the case, you might
         * need to adjust this scaling factor to achieve proper detection.
         */
        sample /= 32768.f;

        /* Queue the sample into Imagimob model */
        if(IMAI_enqueue(&sample) != IMAI_RET_SUCCESS)
        {
            printf("Error while enqueuing sample to model.\n\r");
        }

        /* Check if enough samples are available in model, if yes, get results until empty */
        while(IMAI_dequeue(classifications) == IMAI_RET_SUCCESS)
        {
            /* Only print if classification is confident enough (>70%) */
            if(classifications[1] > 0.70)
            {
                printf("None:\t %.2f Siren:\t %.2f \n\r", classifications[0], classifications[1]);
            }
        }

        /* Keep track of where we currently are in audio stream and print seconds */
        ++samples_sec_cnt;
        if (samples_sec_cnt > SAMPLE_RATE - 1)
        {
            ++seconds;
            printf("[TIME] %d seconds\n\r", (int) seconds);
            samples_sec_cnt = 0;
        }

        /* Keep track of PCM stream and reset to beginning if needed */
        ++pcm_idx;
        if (pcm_idx > PCM_SAMPLES_SIZE - 1)
        {
            pcm_idx = 0;
            seconds = 0;
            samples_sec_cnt = 0;
        }
    }

    return 0;
}

/* [] END OF FILE */
